function U = MeanFlow(x,y,Re,mfac,sweep)
% U = MeanFlow(x,z,Re,mfac,sweep)
% This function computes the mean flow quantities for a Falkner-Skan-Cooke 
% boundary layer. 
% 
% x: array containing coordinates of grid in x-direction
% y: array containing coordinates of grid in z-direction
% Re: Reynolds number at initial x-station
% mfac: Falkner-Skan-Cooke parameter U=C*x^mfac
% sweep: flowsweep angle at initial x-station
%
% Output is the structre array 'U' which contains 
% U(i).u, U(i).v, U(i).w: streamwise, normal and spanwise velocity
% U(i).uy, U(i).vy, U(i).wy: normal derivatives of u, v and w 
% U(i).ux, U(i).wx: streawwise derivatives of u and w 
%
% (c) Ardeshir Hanifi & David Tempelmann, 2014
%

Nstations = length(x);

% generate self-semilar Falkner-Skan-Cooke profiles
Ufsc = similar_fsc(y,mfac);

% generate meanflow data at first station
we=tan(sweep*pi/180);
U(1).u = Ufsc.u;
U(1).v = Ufsc.v/Re;
U(1).w = Ufsc.w*we;
U(1).uy = Ufsc.uy;
U(1).wy = Ufsc.wy*tan(sweep*pi/180);
U(1).vy = Ufsc.vy/Re;
U(1).ux = Ufsc.uy.*( y*(mfac-1)*0.5 )/Re + mfac*Ufsc.u/Re;
U(1).wx = Ufsc.wy.*( y*(mfac-1)*0.5 )*we/Re;

% set global reference scales
method = 'spline';

% rescale meanflow such that all quantities are based on reference scales of
% first station and interpolate onto Gauss-Lobatto Grid
for i = 1:Nstations
    ue = (x(i)/x(1))^mfac;
    lscale = sqrt(x(i)/x(1)/ue);
    rescale = sqrt(ue*x(i)/x(1));
    yi=y*lscale;
    ve = U(1).v(1)*ue/rescale-U(1).ux(1)*ue/rescale*(yi(1)-y(1));
    
    U(i).u=interp1(yi,U(1).u*ue,y,method,ue);
    U(i).w=interp1(yi,U(1).w   ,y,method,we);
    U(i).v=interp1(yi,U(1).v*ue/rescale,y,method,ve);
 
    U(i).uy=interp1(yi,U(1).uy*ue/lscale,y,method,0);
    U(i).wy=interp1(yi,U(1).wy/lscale   ,y,method,0);
    U(i).vy=interp1(yi,U(1).vy*ue/lscale/rescale,y,method,U(1).vy(1)*ue/lscale/rescale);

    U(i).ux=interp1(yi,U(1).ux*ue/rescale^2,y,method,U(1).ux(1)*ue/rescale^2);
    U(i).wx=interp1(yi,U(1).wx/rescale^2   ,y,method,U(1).wx(1)/rescale^2);
    
end
