function [q, alphax, ncalc] = integratePSE(q0, alpha0, x, y, U, D1, D2, W,...
                       beta, omega, Re, Nstation, stab, dpdx, FDorder, iaux)
% [q,alphax,ncalc] = integratePSE(q0, alpha0, x, y, U, D1, D2, W,...
%                           beta, omega, Re, Nstation, stab, dpdx, FDorder, iaux)                   
% Thuis function integrates the Parabolised Stability Equations (PSE).
%
% Input
%    q0, alpha0: complex eigenfunction and stermawise wavenumber at initial station
%    x, y: arrays containing streamwise and normal coordinates of the grid
%    U: structure array containing the meanflow quantities
%    D1, D2, W: first- and second derivative operators and spectral integration weights
%    beta, omega, Re: spanwise wavenumber, frequency and Reynolds number
%    Nstation: number of stations in x-direction to compute
%    stab: if stab=true the stablization of PSE is active
%    dpdx: if =0 the x-derivative of pressure disturbance is neglected
%    FDorder: order of backward Euler discretization scheme (can be 'first' or 'second') 
%    iaux: auxiliary condition is based integral norm if (iaux=1) else on u_max
%
% Output
%    q: complex array contatining eigen functions (u,v,w,p) at computed x-stations
%    alphax: complex array containing streamwise wavenumber at computed x-stations
%    ncalc: number of sucessfully computed stations
%
% (c) Ardeshir Hanifi & David Tempelmann, 2014
%

%initialise variables
N=length(y);
q=zeros(N*4,Nstation);        q(:,1) = q0;
alphax=zeros(Nstation,1);     alphax(1) = alpha0;
alpha = alpha0;
local=false;

dx= [0 diff(x)];
alpha1=0;
auxval1=0;

% integration loop
n=1;
ncalc=Nstation;
 while (n < Nstation) 
    n=n+1;
    disp(' ');
    disp(['Station: ', num2str(n)]);
    
    %decide which FDorder for backward scheme
    if (n == 2 || strcmp(FDorder, 'first'))
        scheme_order = 'first';
    elseif (n ~= 2 && strcmp(FDorder, 'second'))
        scheme_order = 'second';
    end

    %get coefficients for backward scheme
    [eulco] = compeulco(scheme_order, dx(n), dx(n-1));
    
    %loop over alpha to satisfy auxiliary condition
    iloop = 0;
    auxval = 1e8;

    while abs(auxval) > 1e-08 && (iloop<50)
        iloop = iloop + 1;

        %stablization coefficient
        s = 0;
        if stab
            if ( real(alpha) ~= 0  && dx(n) <= 1/abs(real(alpha))*1.01 )
                s = (0.5/abs(real(alpha)) - dx(n)*.5)*2;
            end
            %disp(['stabilisation: s = ' num2str(s)]);
        end
        
        %set up operator matrices with boundary conditions
        [RHS, LHS] = operator(local,U(n),alpha, D1, D2,...
                     beta,omega,Re,eulco,q(:,max(1,n-2)),q(:,n-1),s,dpdx);

        %solve equation system
        q(:,n) = LHS\RHS;
        
        %compute the auxiliary condition
        auxval = auxfunc(eulco, y, q(:,max(1,n-2)), q(:,n-1), q(:,n), W, iaux);
        disp(['It: ', num2str(iloop), ' ,alpha: ', num2str(alpha), ' , auxval: ', num2str(abs(auxval))]);
        alphax(n) = alpha;
        
        %estimate new alpha
        [ alpha, alpha1, auxval1 ] = extrapol( alpha1, alpha, auxval1, auxval, iloop);

    end
    if abs(auxval) > 1e-08
        ncalc=n-1;
        q=q(:,1:ncalc);
        alphax=alphax(1:ncalc);
        n=Nstation+100; 
    end
    
 end
end

