function [RHS,LHS] = operator(local, U, alpha, D1, D2, beta, omega, Re,...
    varargin)

% OPERATOR sets up the operator corrsponding to local and nonlocal stability
% equations.
%
% [RHS,LHS]=OPERATOR(LOCAL,U,ALPHA,D1,D2,BETA,OMEGA,RE,N,EULCO,Q1,Q2,S)
% If LOCAL=false parameters EULCO, Q1, Q2 and S are optional.
%
% Input
%	local: if =true the local operator corresponding to local stability theory is set up
%	U: structure containing meanflow field and its derivatives
%	alpha: streamwise wavenumber
%	D1 and D2: matrices corresponding to first- and second derivative operators
%	beta: spanwise wavenumber
%	omega: angular frequency
%	Re: Reynolds number
%	eulco: coefficients of Euler backward discretization scheme
%	q1 and q2: amplitudefunctions at x(i-1) and x(i-2)
%	s: coefficient of stabilization term
%   dpdx: if =0 the x-derivative of pressure disturbance is neglected
%
% Output
%	LHS and RHS: matrices corresponding to PSE, LHS.q=RHS 
%
% (c) Ardeshir Hanifi & David Tempelmann, 2014
%
%------------------------------------------------------------------
N=length(U.u);

if (local==true) 
    eulco = zeros(3,1);
    q1 = zeros(N*4,1);
    q2 = zeros(N*4,1);
    s = 0;
    dpdx = 0;
elseif (local==false) 
    if (nargin~=13)
        disp('**** OPERATOR: Wrong number of parameters')
        disp('[RHS,LHS] = operator(local,U,alpha,D1,D2,beta,omega,Re,eulco,q1,q2,s)')
        error('Stopping programe');
    else
        eulco = varargin{1};
        q1 = varargin{2};
        q2 = varargin{3};
        s = varargin{4};
        dpdx = varargin{5};
        if (dpdx~=0); dpdx=1; end
    end
end

%------------------------------------------------------------------
% Set up operator matrices
% A.q + B.q_y + C.q_yy + D.q_x=0

if (local==true); inoloc=0; else inoloc=1; end
i = sqrt(-1);
I = eye(size(D1));
O = zeros(size(D1));
xi=diag(-i*omega + (beta^2+alpha^2)/Re + i*(alpha*U.u + beta*U.w));

A11 = i*alpha*I;
A13 = i*beta*I;
A21 = xi+diag(U.ux)*inoloc;
A22 = diag(U.uy);
A24 = i*alpha*I;
A32 = xi+diag(U.vy)*inoloc;
A41 = diag(U.wx)*inoloc;
A42 = diag(U.wy);
A43 = xi;
A44 = i*beta*I;

A = [A11, O,   A13, O  ; ...
     A21, A22, O  , A24; ...
     O,   A32, O  , O  ;
     A41, A42, A43, A44]; ...

%------------------------------------------------------------------

VD1 = diag(U.v)*D1*inoloc;

B = [O,   D1,  O,   O;  ...
     VD1, O,   O,   O;  ...
     O,   VD1, O,   D1; ...
     O,   O,   VD1, O ];

%------------------------------------------------------------------

C = [O,      O,      O,      O; ...
     -D2/Re, O,      O,      O; ...
     O,      -D2/Re, O,      O; ...
     O,      O,      -D2/Re, O];

%------------------------------------------------------------------
if (local)   
    D=zeros(size(A));
else

    u = diag(U.u);
    
    D = [I, O, O, O     ; ...
         u, O, O, I*dpdx; ...
         O, u, O, O     ; ...
         O, O, u, O ];

    D = D + s*(A+B+C); %Include stabilisation term
end

%Left-hand-side and Right-hand-side of equation system
LHS = A + B + C + D*eulco(3);
RHS = -D*(eulco(1)*q1 + eulco(2)*q2);

%---------------------------------------------------------------------
%Boundary conditions
Ov = zeros(1,4*N);

% u(ymax)=0
LHS(N+1,:) = Ov;
LHS(N+1,1) = 1;
RHS(N+1) = 0;

% u(0)=0
LHS(2*N,:) = Ov;
LHS(2*N, N) = 1;
RHS(2*N) = 0;

% v(ymax)=0
LHS(2*N+1,:) = Ov;
LHS(2*N+1, N+1) = 1;
RHS(2*N+1) = 0;

if (local)
    % p(0)=1
    LHS(3*N,:) = Ov;
    LHS(3*N, 4*N) = 1;
    RHS(3*N) = 1.0;
else
    % v(0)=0
    LHS(3*N,:) = Ov;
    LHS(3*N, 2*N) = 1;
    RHS(3*N) = 0;
end

% w(ymax)=0
LHS(3*N+1,:) = Ov;
LHS(3*N+1, 2*N+1) = 1;
RHS(3*N+1) = 0;

% w(0)=0
LHS(4*N,:) = Ov;
LHS(4*N, 3*N) = 1;
RHS(4*N) = 0;

end
