% pse.m is the main program for solving the PSE
% pse.m is the main program to solbe the Parabolized Stability Equations (PSE)
%
% (c) Ardeshir Hanifi & David Tempelmann, 2014
%
%clear all
%% --------------------------------------------------------------------------
% Numerical Parameters
N=100;              % Number of nodes in wall-normal direction
ymax=70;           % Height of last node in wall-normal direction
stab=false;        % if stab=true the stablization of PSE is active
dpdx=1;            % if this parameter is =0 the x-derivative of pressure disturbance is neglected
FDorder='first';   % order of backward Euler discretization scheme (can be 'first' or 'second') 
iaux=1;            % auxiliary condition is based integral norm if (iaux=1) else on u_max
%% --------------------------------------------------------------------------
% Flow parameters
Re=400;            % Reynolds number at first x-station
Re_end=1000;       % Reynolds number at final x-station
Nstation=80;       % Number of x-stations
mfac=0.0;            % Falkner-Skan-Cook patrameter (U=x^mfac)
sweep=0;           % Sweep angle (in degree) at first x-station

%% --------------------------------------------------------------------------
% Perturbation parameters

beta=0.0;     % Spanwise wavenumber
F=1e-4;     % Reduced frequency
omega=F*Re; % Angular frequency

%% --------------------------------------------------------------------------
% Spectral discretisation operators (Chebycheff)

[y,D1,D2,W] = chebmat(N,ymax);

%% --------------------------------------------------------------------------
% Generate meanflow

disp('Computing mean flow')
x = linspace(Re,(Re_end/Re)^(2/(mfac+1))*Re,Nstation);
U = MeanFlow(x,y,Re,mfac,sweep);

%% --------------------------------------------------------------------------
% Solve local Orr-Sommerfeld eigenvalue problem to use as initial solution

alpha0=omega/0.36; % start approximation for TS waves
[q0,alpha0] = localOSS(alpha0,U(1),D1,D2,beta,omega,Re);

% Normalise initial disturbance to energy 1
q0 = q0/(0.5*sqrt(q0(1:3*N)'*blkdiag(W,W,W)*q0(1:3*N)));

% Plot the eigenfunction
c=['k','r','b','g'];
figure(1);clf(1);hold on; for k=1:4;plot(abs(q0((k-1)*N+1:k*N)),y,c(k));end
legend('u','v','w','p')

%% --------------------------------------------------------------------------
% Integrate the Parabolized Stability Equations

disp('Integrate Parabolized Stability Equations')

[q, alpha, ncalc] = integratePSE(q0, alpha0, x, y, U, D1, D2, W,...
    beta, omega, Re, Nstation, stab, dpdx, FDorder, iaux);
x=x(1:ncalc);

% Compute growth rate
sigma=growth( x,alpha,q,W );

% Compute N-factor
Nfactor=cumtrapz(x,sigma);

% Plot growth rate (scaled globally) as function of local Re
Rex=(x/x(1)).^((mfac+1)/2)*Re;
figure(2);clf(2); plot(Rex,sigma);
grid on; xlabel('Re'); ylabel('growth rate');

d=([Rex' sigma]);
save('sigma.dat','d','-ascii','-double')
break
%% --------------------------------------------------------------------------
%  
% This part of code is an example of how the local stability results could
% be obtained.
beta=0;     % Spanwise wavenumber
F=1e-4;     % Reduced frequency

Rex=(400:20:800);
alpha_l=zeros(size(Rex));
for k=1:length(Rex)
omega=F*Rex(k); % Angular frequency
alpha0=omega/0.36; % start approximation for TS waves
[q0,alpha_l(k)] = localOSS(alpha0,U(1),D1,D2,beta,omega,Rex(k));
end

% plot results scaled by local refernce values
figure(3);clf(3); plot(Rex,-imag(alpha_l));grid on;



