function [ A, B ] = form_EVP2d( Nx, Ny, Re, xgl, d1x, d2x, d1y, d2y)
%
% FORM_EVP2D: Form the Linearized Navier-Stokes Equations in 2D
%
% INPUTS
% Nx:         number of Fourier collocation nodes
% d1x, d2x:   Fourier collocation derivative matrices
% Ny:         number of Chebyshev collocation nodes
% xgl:        Chebyshev Gauss Lobatto nodes
% d1y, d2y:   Chebyshev Gauss Lobatto collocation derivative matrices
% U, Uy:      Streamwise velocity of plane Poiseuille flow and 1st derivative 
%
% OUTPUTS
% A:          LHS of the discretized LNSE operator in 2D
% B:          RHS of the discretized LNSE operator in 2D,
%             including an artificial compressibility parameter: 
%             epsilon=1e-8
%
% (c) 2014 vt_productions

Ix=eye(Nx);
Iy=eye(Ny+1);

U=1.-xgl.^2;
Uy=-2*xgl;

epsilon = 1e-8;

Lop=1/Re*(kron(d2x,Iy)+kron(Ix,d2y))-kron(Ix,diag(U))*kron(d1x,Iy);

A11=Lop;
A12=-kron(Ix,diag(Uy));
A13=-kron(d1x,Iy);
A1=horzcat(A11,A12,A13);

A21=zeros(Nx*(Ny+1));
A22=Lop;
A23=-kron(Ix,d1y);
A2=horzcat(A21,A22,A23);

A31=kron(d1x,Iy);
A32=kron(Ix,d1y);
A33=zeros(Nx*(Ny+1));
A3=horzcat(A31,A32,A33);

A=vertcat(A1,A2,A3);

B11=eye(Nx*(Ny+1));
B12=zeros(Nx*(Ny+1));
B13=zeros(Nx*(Ny+1));
B1=horzcat(B11,B12,B13);

B21=zeros(Nx*(Ny+1));
B22=eye(Nx*(Ny+1));
B23=zeros(Nx*(Ny+1));
B2=horzcat(B21,B22,B23);

B31=zeros(Nx*(Ny+1));
B32=zeros(Nx*(Ny+1));
%B33=zeros(Nx*(Ny+1));
B33=epsilon*eye(Nx*(Ny+1));
B3=horzcat(B31,B32,B33);

B=vertcat(B1,B2,B3);

return